package com.persagy.job;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.persagy.cache.AlarmInfoCache;
import com.persagy.cache.CreatedAlarmIdsCache;
import com.persagy.client.GroupNettyClient;
import com.persagy.entity.AlarmDefineState;
import com.persagy.entity.AlarmRecord;
import com.persagy.entity.NettyMessage;
import com.persagy.entity.ZktAlarmRecordDO;
import com.persagy.repository.AlarmRecordRepository;
import com.persagy.utils.DateUtils;
import com.persagy.utils.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.util.Collections;
import java.util.Date;
import java.util.concurrent.atomic.AtomicLong;

import static org.quartz.SimpleScheduleBuilder.simpleSchedule;

@DisallowConcurrentExecution
@PersistJobDataAfterExecution
@Slf4j
public class AlarmExpireJob extends QuartzJobBean {

    @Autowired
    private GroupNettyClient groupNettyClient;
    @Autowired
    AlarmRecordRepository alarmRecordRepository;
    @Autowired
    AlarmInfoCache alarmInfoCache;
    @Autowired
    CreatedAlarmIdsCache createdAlarmIdsCache;
    /**
     * 报警记录信息详情
     */
    private String alarmRecord;
    /**
     * 重试次数
     */
    private String refire;
    /**
     * 过期时间
     */
    private String expireTime;
    /**
     * 报警定义标识（itemcode+objId）
     */
    private String defineId;

    public String getState() {
        return state;
    }

    public void setState(String state) {
        this.state = state;
    }

    /**
     * 报警状态（2-恢复 3-过期）
     */
    private String state;

    public String getEndTime() {
        return endTime;
    }

    public void setEndTime(String endTime) {
        this.endTime = endTime;
    }

    public String getEndInfo() {
        return endInfo;
    }

    public void setEndInfo(String endInfo) {
        this.endInfo = endInfo;
    }

    /**
     * 恢复时间
     */
    private String endTime;
    /**
     * 恢复时刻信息
     */
    private String endInfo;
    /**
     * 集团编码
     */
    @Value("${group.code}")
    public String groupCode;

    private AtomicLong nums = new AtomicLong(1L);

    /**
     * @description: 执行恢复/过期定时任务
     * @param: context
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2021/1/22 上午9:38
     * @version: V1.0
     */
    @Override
    protected void executeInternal(JobExecutionContext context) throws JobExecutionException {
        try {
            String jobType = "2".equals(state) ? "恢复" : "过期";
            JobDataMap mergedJobDataMap = context.getMergedJobDataMap();
            log.info("定时任务开始{}", alarmRecord);
            log.info("定时任务为{}任务", jobType);
            log.warn("refireCount:[{}],过期/恢复时间：[{}/{}],实际执行时间：[{}]",
                    context.getRefireCount(),
                    StringUtil.getString(expireTime),
                    StringUtil.getString(endTime),
                    DateUtil.format(context.getFireTime(), DatePattern.ISO8601_PATTERN));
            if (StringUtils.isNotBlank(alarmRecord)) {
                ZktAlarmRecordDO zktAlarmRecordDO = StringUtil.tranferItemToDTO(alarmRecord, ZktAlarmRecordDO.class);
                String alarmId = zktAlarmRecordDO.getAlarmId();

                // 立即过期的场景，过期的时候可能还没有报警记录ID,需要延迟重新执行
                if (!createdAlarmIdsCache.contains(alarmId)) {
                    log.info("未获取到报警记录id, 定时任务将在5分钟后重新执行");
                    log.info("refire:[{}]", refire);
                    mergedJobDataMap.put("refire", String.valueOf(StringUtil.getInt(refire) + 1));
                    reFireJob(context, mergedJobDataMap);
                    log.info("----------------结束---------------------");
                    return;
                }

                log.info("报警参数为：[{}]", zktAlarmRecordDO.toString());
                NettyMessage<AlarmRecord> nettyMessage = new NettyMessage<>(6, zktAlarmRecordDO.getProjectId());
                nettyMessage.setStreamId(nums.getAndIncrement());
                AlarmRecord message = AlarmRecord.builder()
                        .id(alarmId)
                        .state(StringUtil.getInt(state))
                        .groupCode(groupCode)
                        .projectId(zktAlarmRecordDO.getProjectId())
                        .build();
                if ("2".equals(state)) {
                    message.setEndInfo(endInfo);
                    message.setEndTime(DateUtils.parseDate(endTime));
                }
                nettyMessage.setContent(Collections.singletonList(message));
                //{"id","123", "state":1, "groupCode":"wd", "projectId":"Pj123"}
                log.info("定时任务执行完成，向云端推送{}消息", jobType);
                groupNettyClient.sendMessage(nettyMessage.toString());

                // 删除缓存中报警id
                createdAlarmIdsCache.remove(alarmId);

                //已经过期的时候删除掉这条报警定义了，保证不会再次产生报警
                AlarmDefineState alarmState = new AlarmDefineState(defineId);
                alarmState.setAlarmStartTime("");
                alarmInfoCache.setAlarmState(defineId, alarmState);
                if (alarmRecordRepository.existsById(zktAlarmRecordDO.getDefinitionId())) {
                    alarmRecordRepository.deleteById(zktAlarmRecordDO.getDefinitionId());
                }
            }
            log.info("----------------结束---------------------");
        } catch (Exception e) {
            log.error("job hander error", e);
        }

    }


    private void reFireJob(JobExecutionContext context, JobDataMap mergedJobDataMap) {
        try {
            if (StringUtil.getInt(refire) > 1) {
                log.error("重新执行依然失败,信息为：{}[{}]", state, alarmRecord);
            } else {
                Scheduler scheduler = context.getScheduler();
                Trigger trigger = context.getTrigger();
                DateTime dateTime = DateUtil.offsetSecond(new Date(), 300);
                Trigger newTrigger = TriggerBuilder.newTrigger()
                        .startAt(dateTime.toJdkDate())
                        .withIdentity(trigger.getKey())
                        .usingJobData(mergedJobDataMap)
                        .withSchedule(simpleSchedule().withMisfireHandlingInstructionIgnoreMisfires()).build();
                Date rescheduleJob = scheduler.rescheduleJob(trigger.getKey(), newTrigger);
            }
        } catch (Exception e) {
            log.error("获取不到报警记录ID.重新获取报错！", e);
        }
    }

    public String getAlarmRecord() {
        return alarmRecord;
    }

    public void setAlarmRecord(String alarmRecord) {
        this.alarmRecord = alarmRecord;
    }

    public String getRefire() {
        return refire;
    }

    public void setRefire(String refire) {
        this.refire = refire;
    }

    public String getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(String expireTime) {
        this.expireTime = expireTime;
    }

    public String getDefineId() {
        return defineId;
    }

    public void setDefineId(String defineId) {
        this.defineId = defineId;
    }
}
